#ifndef DC_SETTINGS_H
#define DC_SETTINGS_H

#include "hub_info.h"

#include <set>
#include <map>
#include <qstring.h>

using namespace std;

class FavData
{
public:
     FavData(const QString& aIp, const QString& aPwd) : ip(aIp), pwd(aPwd) {}
     QString ip;
     QString pwd;
};

typedef map<int,QString> Map;
typedef map<QString,FavData> StrMap;

class QStringList;

class dc_settings
{
public:
  enum {NICKNAME,DESCRIPTION,EMAIL,CONNECTION,SLOTS,ADDRESS,ENCODING,ALLOWFILELIST,DLIMIT,DESCTAG,ABORT_LEAVED,
	   PORT,PASSIVE,ULIMIT,OFFSET,DLDIR,FONT,PROXYADDRESS,PROXYPORT,PROXYUSERNAME,PROXYPWD,USEPROXY,USESOCKS};

     enum {NO_SETTINGS,ERROR_LOADING_PREFS,ERROR_LOADING_FAVOURITES,SUCCESS};

     dc_settings();

     int init();

     bool load_from_file();
     bool load_favourites_file();
     bool save_to_file();
     bool save_favourites_file();

     void add_share(const QString &dir);
     void del_share(const QString &dir);
     void clear_shares();

     void add_favourite(const QString &name,const QString &address,const QString &password);
     void del_favourite(const QString &name);

     int num_favourites() const {return favourites.size();}
     bool get_favourite(int i,QString &name,QString &address,QString &password);
     const hub_info get_favourite(const QString &name);

     QStringList get_connection_types();

     QString get_setting(int key);
     void set_setting(int key,const QString &value);

#ifdef _DEBUG
     void print();
#endif

     int num_shares() const {return shares.size();}
     const QString get_share(int i);
     const QString get_home_dir() const { return user_home_dir; }

private:
     int get_setting_key(const QString &s) const;

     QString settings_file;
     QString favourites_file;

     set<QString> shares;
     Map settings;
     StrMap favourites;
     QString user_home_dir;
};

extern dc_settings settings;

#endif
