#ifndef _MAINWDGT_H_
#define _MAINWDGT_H_

#include <qtabwidget.h>
#include <qpopupmenu.h>
#include <qlineedit.h>
#include <qmainwindow.h>
//#include "hubwdgt.h"
//#include "debugdlg.h"

//class TransferDlg;
class transfer_widget;
class QAction;
class QNetworkOperation;
class QProgressBar;
class QProgressDialog;
class HubWdgt;

class MainWdgt : public QMainWindow
{

	Q_OBJECT

public:
	MainWdgt( QWidget* parent = 0, const char* name = 0 );

	/// Returns a pointer to the instance of MainWdgt.
	static const MainWdgt& getInstance();

	/// Returns the number of connected hubs.
	int getNumConnections() const;

public slots:

	/*!
	  Slot that displays a simplae dialog with
	  info about the application
	 */
	void displayAbout();

	/*!
	  Slot that displays the about qt dialog.
	 */
	void displayAboutQt();

	/// Slot for showing the online help
	void displayHelp();


	/*!
	  Slot for making a search in all hubs that we are connected to.
	  Retreives the search string and then emits a search signal with
	  the search string so the hub widgets can perform the search.
	*/
	void search(){}

	/*!
	  Slot for creating a new hub tab, used to connet to a hub.
	*/
	void newHubTab();

	/*!
	  Slot for removing a hub tab when the user has pressed the close
	  button

	 */
	void closeHubTab( );

	/*!
	  Brings up the preferences dialog.
	*/
	void showPreferences();

	void setHubName(const QString &name, QWidget *wdgt);

	/// Connects the current hub
	void connectHub();

	/// Disconnects the current hub
	void disHub();

	/// Brings up the search dialog
	void searchHub();

	void selectHub(QWidget*);

	// Slot for handling multihub searh results
	void dispatchSearchResult(const QString& u,const QString& f,const QString& s,const QString& sl,const QString& hubname);
	void hubDisconnected(int id);
	void hubConnected(int id);

	// Version retrieval slots
	void checkVersion();
	void incomingData(const QByteArray &data, QNetworkOperation *op);
	void versionCheckFinish(QNetworkOperation *op);
	void dctcOutputReady();

	/// Shows a message in the status bar
	void showStatus(const QString&);

	/// Shows some progrss in the status bar
	void showProgress(const QString&,const QString&);

	/// Enables or disables the actions
	void enableActions(bool yes);

	/// Rebuilds the shared database
	void reshare();

signals:
	/*!
	  Signal that is emitted when a user wants to search the hubs
	  we are connected to at the moment
	 */
	void search( const QString& str );

private:
	/*!
	  Line edit for entering search strings
	 */
	QLineEdit* searchEdit;

	/*!
	  The tab widget that holds our hub tabs
	 */
	QTabWidget* hubTabs;

	/*!
	  The transfer dialog
	*/
	// TransferDlg* transferWin;

	/*!
	  The new and improved transfer widget.
	*/
	transfer_widget* transferWidget;

	virtual void closeEvent( QCloseEvent *e);
	void checkDCTCVersion();

	QAction *aNewHub;
	QAction *aCloseHub;
	QAction *aConnect;
	QAction *aDisconnect;
	QAction *aSearch;
	QAction *aConfig;
	QAction *aExit;
	QAction *aReshare;

	QByteArray versionData;
	QString dctcVersion;
//	QProgressBar *progressBar;
	QProgressDialog *progressDlg;

	static MainWdgt* instance;

};

#endif

/*
 *
 * $Log: mainwdgt.h,v $
 * Revision 1.10  2004/01/09 22:02:54  olof
 * Fixed used description bug and added support for description tags.
 * There is still some work to be done though.
 *
 * Revision 1.9  2004/01/07 20:25:23  olof
 * bugfixes
 *
 * Revision 1.8  2004/01/06 17:37:38  olof
 * bufixes!!
 *
 * Revision 1.7  2003/12/20 18:38:57  olof
 * Major bugfixes
 *
 * Revision 1.6  2003/10/05 22:36:49  olof
 * fixes
 *
 * Revision 1.5  2003/08/19 17:37:22  olof
 * version checker
 *
 * Revision 1.4  2003/08/18 13:06:19  olof
 * Version checker
 *
 * Revision 1.3  2003/08/18 12:19:43  estrato
 * Old transfer dialog now completely replaced, but some functionality is gone. Will add complete functionality in next commit.
 *
 *
 */
