/*
  Copyright 2003 Rikard Bjrklind, Mikael Gransell

  This file is part of dc-qt.

  dc-qt is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  dc-qt is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with dc-qt; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "preferences_dlg.h"
#include "dc_settings.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qhgroupbox.h>
#include <qvgroupbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlistview.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include <qfontdialog.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qtabwidget.h>
#include <qvbuttongroup.h>
#include <qradiobutton.h>

using namespace std;

preferences_dlg::preferences_dlg(QWidget *p,
			     const char *name,
			     bool modal)
  : QDialog(p, name, modal)
{
     setCaption("dc-qt preferences");
	bool ok;        // used for conversion checking
  
	QTabWidget *tabs = new QTabWidget(this,"muu");
	
	QGridLayout *grid = new QGridLayout(this,4,1);
  
	QHBoxLayout *bottom = new QHBoxLayout;
	grid->addLayout(bottom,3,0);
	QPushButton *okBtn = new QPushButton("&OK",this);
	QPushButton *cancelBtn = new QPushButton("&Cancel",this);
//	QPushButton *fontBtn = new QPushButton("Choose &font",this);
	bottom->addWidget(okBtn);
	bottom->addWidget(cancelBtn);
//	bottom->addWidget(fontBtn);
	connect(okBtn,SIGNAL(clicked()),this,SLOT(update_settings()));
	connect(okBtn,SIGNAL(clicked()),this,SLOT(close()));
	connect(cancelBtn,SIGNAL(clicked()),this,SLOT(close()));
//	connect(fontBtn,SIGNAL(clicked()),this,SLOT(chooseFont()));

//	QGroupBox *userbox = new QGroupBox("User",this,"user box");
	QFrame *userbox = new QFrame(this,"user box");

	QGroupBox *connbox = new QGroupBox("Connection",this,"connection box");
	QVGroupBox *dirbox = new QVGroupBox("Sharing",this,"dir box");
	QVGroupBox *guibox = new QVGroupBox("User interface",this,"guibox");
	QGroupBox *netbox = new QGroupBox(10,Qt::Vertical,"Network",this,"network box");

	// Name
	QGridLayout *userGrid = new QGridLayout(userbox,3,3,10);
	userGrid->setRowStretch(3,10);
						
	QLabel *editLbl = new QLabel("Name:", userbox);
	nameEdit = new QLineEdit(settings.get_setting(dc_settings::NICKNAME),
							userbox,"nameedit");
	// Description
	QLabel *descLbl = new QLabel("Description:",userbox);
	descEdit = new QLineEdit(settings.get_setting(dc_settings::DESCRIPTION),
							userbox,"descedit");
	// Email
	QLabel *emailLbl = new QLabel("Email:", userbox);
	emailEdit = new QLineEdit(settings.get_setting(dc_settings::EMAIL),userbox,"emailedit");

	userGrid->addWidget(editLbl,0,0);
	userGrid->addWidget(nameEdit,0,1);
	userGrid->addWidget(descLbl,1,0);
	userGrid->addWidget(descEdit,1,1);
	userGrid->addWidget(emailLbl,2,0);
	userGrid->addWidget(emailEdit,2,1);

	// -------------- Network section -------------------
	// Slots
	QLabel *slotsLbl = new QLabel("Upload slots:",netbox);
	int slotnr = settings.get_setting(dc_settings::SLOTS).toInt(&ok);
	slotsSpin = new QSpinBox(1,15,1,netbox);
	slotsSpin->setValue(slotnr);	
	
	// Max upload speed
	QLabel *speedLbl = new QLabel("Max upload speed (kb/s):",netbox);
	int limit = settings.get_setting(dc_settings::ULIMIT).toInt(&ok);
	speedSpin = new QSpinBox(0,99999,1,netbox);
	speedSpin->setValue(limit);

	// Max download speed
	QLabel *dspeedLbl = new QLabel("Max download speed (kb/s):",netbox);
	int dlimit = settings.get_setting(dc_settings::DLIMIT).toInt(&ok);
	dspeedSpin = new QSpinBox(0,99999,1,netbox,"hooo");
	dspeedSpin->setValue(dlimit);

	desctagButtons = new QVButtonGroup("Add description tag",netbox,"desctagbuttons");
	new QRadioButton("None",desctagButtons);
	new QRadioButton("DCTC",desctagButtons);
	new QRadioButton("DCGUI",desctagButtons);
	new QRadioButton("dc++",desctagButtons);
	QString desctag = settings.get_setting(dc_settings::DESCTAG);
	if(desctag=="none") desctagButtons->setButton(0);
	else if(desctag=="dctc") desctagButtons->setButton(1);
	else if(desctag=="dcgui") desctagButtons->setButton(2);
	else if(desctag=="dc++") desctagButtons->setButton(3);

	// Grant filelist slot option
	QHBox *granthbox = new QHBox(netbox);
	bool allowflst = settings.get_setting(dc_settings::ALLOWFILELIST)=="true"?true:false;
	fileListCheck = new QCheckBox(granthbox,"allowcheckbox");
	fileListCheck->setChecked(allowflst==1);
	new QLabel("Always allow file list upload",granthbox);


	// Abort download when user leaves hub checkbox
	QHBox *abortbox = new QHBox(netbox);
	bool abortleaved = settings.get_setting(dc_settings::ABORT_LEAVED)=="true"?true:false;
	abortleavedCheck = new QCheckBox(abortbox,"abortcheck");
	abortleavedCheck->setChecked(abortleaved);
	new QLabel("Abort download when user leaves hub",abortbox);

	// ------------------- Connection section --------------------

	QGridLayout *netGrid = new QGridLayout(connbox,12,4,14);

	// Port
	QLabel *portLbl = new QLabel("Port:",connbox);
	int portnr = settings.get_setting(dc_settings::PORT).toInt(&ok);
	portSpin = new QSpinBox(0,65535,1,connbox);
	portSpin->setValue(portnr);

	// Address
	QLabel *addrLbl = new QLabel("Forced IP (leave blank for default):",connbox);
	addrEdit = new QLineEdit(settings.get_setting(dc_settings::ADDRESS),
			   connbox,"Address");

	// Connection type, sorry for the ugly code.
	connectionCombo = new QComboBox(connbox,"connectiontype");
	QStringList connTypes = settings.get_connection_types();
	int connTypeIndex = connTypes.findIndex(settings.get_setting(dc_settings::CONNECTION));
	connectionCombo->insertStringList( connTypes );
	connectionCombo->setCurrentItem(connTypeIndex);
  
	// passive
	passive = new QCheckBox("Passive connection",connbox);
	if(settings.get_setting(dc_settings::PASSIVE) == "true")
		passive->setChecked(true);

	// Proxy stuff
	proxyCheck = new QCheckBox("Use proxy",connbox);
	if(settings.get_setting(dc_settings::USEPROXY)=="true") proxyCheck->setChecked(true);
	QLabel *proxyAddrLbl = new QLabel("Proxy address:",connbox);
	proxyAddr = new QLineEdit(settings.get_setting(dc_settings::PROXYADDRESS),connbox,"proxyaddress");
	QLabel *proxyPortLbl = new QLabel("Port:",connbox);
	proxyPort = new QLineEdit(settings.get_setting(dc_settings::PROXYPORT),connbox,"proxyport");
	if(proxyPort->text() == QString::null)
	     proxyPort->setText("8080");
	proxyUseSocks = new QCheckBox("Use SOCKS",connbox);
	if(settings.get_setting(dc_settings::USESOCKS)=="true") proxyUseSocks->setChecked(true);
	QLabel *proxyUserLbl = new QLabel("Username:",connbox);
	proxyUsername = new QLineEdit(settings.get_setting(dc_settings::PROXYUSERNAME),connbox,"proxyusername");
	QLabel *proxyPwdLbl = new QLabel("Password:",connbox);
	proxyPassword = new QLineEdit(settings.get_setting(dc_settings::PROXYPWD),connbox,"proxypassword");
	
	if(!proxyCheck->isChecked())
	     disableProxyWidgets(!true);
	if(!proxyUseSocks->isChecked())
	     disableSocksWidgets(!true);

	connect(proxyCheck,SIGNAL(toggled(bool)),this,SLOT(disableProxyWidgets(bool)));
	connect(proxyUseSocks,SIGNAL(toggled(bool)),this,SLOT(disableSocksWidgets(bool)));

	netGrid->addWidget(portLbl,0,0);
	netGrid->addWidget(portSpin,0,1);
	netGrid->addWidget(addrLbl,3,0);
	netGrid->addWidget(addrEdit,3,1);
	netGrid->addWidget(connectionCombo,4,1);
	netGrid->addWidget(passive,5,1);
	netGrid->addWidget(proxyCheck,6,0);
	netGrid->addWidget(proxyAddrLbl,7,0);
	netGrid->addWidget(proxyAddr,7,1);
	netGrid->addWidget(proxyPortLbl,8,0);
	netGrid->addWidget(proxyPort,8,1);
	netGrid->addWidget(proxyUseSocks,9,0);
	netGrid->addWidget(proxyUserLbl,10,0);
	netGrid->addWidget(proxyUsername,10,1);
	netGrid->addWidget(proxyPwdLbl,11,0);
	netGrid->addWidget(proxyPassword,11,1);
			   

	// Folders section TODO DnD
	// shared
	shared = new QListView(dirbox,"sharedlist");
	shared->addColumn("Directory");
  
	for(int i=0;i < settings.num_shares();i++)
		QListViewItem *directory = new QListViewItem(shared,settings.get_share(i));

	QHBox *hbox1 = new QHBox(dirbox);

	QPushButton *add = new QPushButton("add",hbox1);
	QPushButton *remove = new QPushButton("remove",hbox1);

	connect(add,SIGNAL(clicked()),this,SLOT(addSharesDlg()));
	connect(remove,SIGNAL(clicked()),this,SLOT(removeSelectedShare()));

	// DL path
	QHBox *hbox2 = new QHBox(dirbox);
	QLabel *dlLbl = new QLabel("Download path:",hbox2);
	dlEdit = new QLineEdit(settings.get_setting(dc_settings::DLDIR),hbox2,"dlpath"); 

	QPushButton *dlBrowseBtn = new QPushButton("...",hbox2);
	connect(dlBrowseBtn,SIGNAL(clicked()),this,SLOT(openDirSelDlg()));

	// GUI tab section
	QHGroupBox *transbox = new QHGroupBox("Text encoding",guibox);
	encodingBox = new QComboBox(transbox,"translationcodecscombo");
	
	encodingBox->insertItem( "Latin1");
	encodingBox->insertItem( "Big5 -- Chinese");
	encodingBox->insertItem( "Big5-HKSCS -- Chinese");
	encodingBox->insertItem( "eucJP -- Japanese");
	encodingBox->insertItem( "eucKR -- Korean");
	encodingBox->insertItem( "GB2312 -- Chinese");
	encodingBox->insertItem( "GBK -- Chinese");
	encodingBox->insertItem( "GB18030 -- Chinese");
	encodingBox->insertItem( "JIS7 -- Japanese");
	encodingBox->insertItem( "Shift-JIS -- Japanese");
	encodingBox->insertItem( "TSCII -- Tamil");
	encodingBox->insertItem( "utf8 -- Unicode, 8-bit");
	encodingBox->insertItem( "utf16 -- Unicode");
	encodingBox->insertItem( "KOI8-R -- Russian");
	encodingBox->insertItem( "KOI8-U -- Ukrainian");
	encodingBox->insertItem( "ISO8859-1 -- Western");
	encodingBox->insertItem( "ISO8859-2 -- Central European");
	encodingBox->insertItem( "ISO8859-3 -- Central European");
	encodingBox->insertItem( "ISO8859-4 -- Baltic");
	encodingBox->insertItem( "ISO8859-5 -- Cyrillic");
	encodingBox->insertItem( "ISO8859-6 -- Arabic");
	encodingBox->insertItem( "ISO8859-7 -- Greek");
	encodingBox->insertItem( "ISO8859-8 -- Hebrew, visually ordered");
	encodingBox->insertItem( "ISO8859-8-i -- Hebrew, logically ordered");
	encodingBox->insertItem( "ISO8859-9 -- Turkish");
	encodingBox->insertItem( "ISO8859-10");
	encodingBox->insertItem( "ISO8859-13");
	encodingBox->insertItem( "ISO8859-14");
	encodingBox->insertItem( "ISO8859-15 -- Western");
	encodingBox->insertItem( "IBM 850");
	encodingBox->insertItem( "IBM 866");
	encodingBox->insertItem( "CP874");
	encodingBox->insertItem( "CP1250 -- Central European");
	encodingBox->insertItem( "CP1251 -- Cyrillic");
	encodingBox->insertItem( "CP1252 -- Western");
	encodingBox->insertItem( "CP1253 -- Greek");
	encodingBox->insertItem( "CP1254 -- Turkish");
	encodingBox->insertItem( "CP1255 -- Hebrew");
	encodingBox->insertItem( "CP1256 -- Arabic");
	encodingBox->insertItem( "CP1257 -- Baltic");
	encodingBox->insertItem( "CP1258");
//	encodingBox->insertItem( "Apple Roman");
	encodingBox->insertItem( "TIS-620 -- Thai");

	// Find the item in the box that matches the loaded setting and select it
	QString enc = settings.get_setting(dc_settings::ENCODING);
	for(int i=0;i < encodingBox->count();i++)
	     if( encodingBox->text(i).section(" ",0,0) == enc )
	     {
		  encodingBox->setCurrentItem(i);
		  break;
	     }
	

	QHGroupBox *fontbox = new QHGroupBox("Application font",guibox);
	fontEdit = new QLineEdit(settings.get_setting(dc_settings::FONT),fontbox,"fontlineedit");
	fontEdit->setReadOnly(true);
	QPushButton *fontBtn = new QPushButton("...",fontbox,"fontchoosebtn");
	connect( fontBtn, SIGNAL(pressed()), this, SLOT(chooseFont()));

	tabs->addTab(connbox,"Co&nnection");
	tabs->addTab(netbox,"N&etwork");
	tabs->addTab(userbox,"&User");
	tabs->addTab(dirbox,"&Directories");
	tabs->addTab(guibox,"&Interface");
	grid->addWidget(tabs,0,0);	
}

void preferences_dlg::openDirSelDlg()
{
	QString dir = QFileDialog::getExistingDirectory(settings.get_home_dir(), this, "dldir_dialog",
													"Select download directory");
	dlEdit->setText(dir);
}

void preferences_dlg::removeSelectedShare()
{
	shared->takeItem( shared->selectedItem() );
}

void preferences_dlg::addSharesDlg()
{
	QString dir = QFileDialog::getExistingDirectory(settings.get_home_dir(), this, "shdir_dialog",
													"Select a sharing directory");

	if(dir != QString::null)
		QListViewItem *item = new QListViewItem(shared,dir);
}

void preferences_dlg::chooseFont()
{
	bool ok;
	font = QFontDialog::getFont( &ok, QApplication::font(), this );
	if ( ok ) {
	// Set tahe desired font
	QApplication::setFont(font,true);
	fontEdit->setText( font.toString() );
	} else {
	// Font dialog was cancelled.
    }
}

//  update_settings()
//   Typically called when someone presses OK.
//   writes all config values to the preferences object,
void preferences_dlg::update_settings()
{
	settings.set_setting( dc_settings::NICKNAME, nameEdit->text() );
	settings.set_setting( dc_settings::DESCRIPTION, descEdit->text() );
	settings.set_setting( dc_settings::EMAIL, emailEdit->text() );
	settings.set_setting( dc_settings::ADDRESS, addrEdit->text() );
	settings.set_setting( dc_settings::SLOTS, slotsSpin->text() );
	settings.set_setting( dc_settings::PORT, portSpin->text() );
	settings.set_setting( dc_settings::CONNECTION, connectionCombo->currentText() );
	settings.set_setting( dc_settings::PASSIVE, passive->isChecked() ? "true" : "false");
	settings.set_setting( dc_settings::ALLOWFILELIST, fileListCheck->isChecked()?"true":"false");
	settings.set_setting( dc_settings::ULIMIT, speedSpin->text() );
	settings.set_setting( dc_settings::DLIMIT, dspeedSpin->text());
	settings.set_setting( dc_settings::DLDIR, dlEdit->text() );
	settings.set_setting( dc_settings::FONT, font.toString() );

	settings.set_setting( dc_settings::PROXYADDRESS, proxyAddr->text() );
	settings.set_setting( dc_settings::PROXYPORT, proxyPort->text() );
	settings.set_setting( dc_settings::PROXYUSERNAME, proxyUsername->text() );
	settings.set_setting( dc_settings::PROXYPWD, proxyPassword->text() );
	settings.set_setting( dc_settings::USEPROXY, proxyCheck->isChecked() ? "true" : "false");
	settings.set_setting( dc_settings::USESOCKS, proxyUseSocks->isChecked() ? "true" : "false");
	//settings.set_setting( dc_settings::FAKE_DCPP, fakedcppCheck->isChecked() ? "true":"false");
	QString desctag;
	switch(desctagButtons->id(desctagButtons->selected())) {
		case 0:
			desctag = "none";
			break;
		case 1:
			desctag = "dctc";
			break;
		case 2:
			desctag = "dcgui";
			break;
		case 3:
			desctag = "dc++";
			break;
		default:
			;
	}
	settings.set_setting( dc_settings::DESCTAG,desctag);
	settings.set_setting( dc_settings::ABORT_LEAVED, abortleavedCheck->isChecked() ? "true":"false");

	settings.set_setting( dc_settings::ENCODING, encodingBox->currentText().section(" ",0,0) );

	// Add shares
	settings.clear_shares();
	QListViewItem *item = shared->firstChild();
	while(item) 
	{
		settings.add_share(item->text(0));
		item = item->nextSibling();
	}

	if(!settings.save_to_file())
	     QMessageBox::critical(this, "Error", "Error saving the preferences file.","Ok");
}

void preferences_dlg::disableProxyWidgets(bool yes)
{
     proxyAddr->setDisabled(!yes);
     proxyPort->setDisabled(!yes);
     proxyUseSocks->setDisabled(!yes);
     if(yes)
	  disableSocksWidgets(proxyUseSocks->isChecked());
     else disableSocksWidgets(false);

}

void preferences_dlg::disableSocksWidgets(bool yes)
{
     proxyUsername->setDisabled(!yes);
     proxyPassword->setDisabled(!yes);
}
